#################################################################################
#                                                                               #
#  65C02 Simulator for PI  v1.0                             Feb 23, 2018        #
#                                                                               #
#  Written by Daryl Ritor with credit to David Welch's awesome tutorials        #
#                                                                               #
#################################################################################
#
#  Main Simulation Program, CPUinit is called from bios.s
#

#  R0, R1, R2 = temp registers - R0= address, R1 = data, R2 = temp for 16bit access

rpc .req  r3			@ PC register
Acc .req  r4			@ Acc register
x   .req  r5			@ X register
y   .req  r6			@ Y register
rsp .req  r7			@ Stack Pointer register (8 bit)
rsr .req  r8			@ Status register
int .req  r9			@ interrupt control register (1=NMI, 2=IRQ, 4=WAI)
iobase .req r10			@ holds IO base address
jbase .req r11			@ jmp table base

.include "macros.inc"		@ file holding actual code for simulation in a series of macro calls

#
# CPUinit is called from bios.s after the hardware system is initialized
# This will initialize the memory system and registers and begin 65C02 execution
# 
.globl CPUinit
CPUinit:
   ldr jbase, =jmp		@ speed up fetch routine
#
# The iobase register can be changed to select where the IO page is located.  If you want 48K RAM,
# set iopage to 0xC0 and you will have 48k RAM and 15.75k of ROM.
#
   mov iobase, #0x80		@ 0x80xx is the io page
   bl mem_init			@ initialize memory
   RESET			@ Reset the CPU state
   FETCH			@ start execution

#
#  NMI interrupt support code (future)
#
nmi:
   mov r0, rpc 
   pushs16			@ return address
   orr r0, rsr, #0x30
   pushs			@ save status w/ break bit set	
   and rsr, rsr, #0xF7		@ clear D flag
   mov rpc, #0xFA
   orr rpc, rpc, #0xFF00	@ r0 = FFFA (NMI vector address)
   readpc16			
   ldrb r1, [r0]		@ get opcode in r1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

#
#  IRQ interrupt support code (future)
#
irq:
   mov r0, rpc 
   pushs16			@ return address 
   and r0, rsr, #0xEF		@ clear B bit
   orr r0, r0, #0x20		@ set unused bit
   pushs			@ save status w/ break bit clear	
   and rsr, rsr, #0xF7		@ clear D flag
   orr rsr, rsr, #0x34		@ set I flag
   mov rpc, #0xFE
   orr rpc, rpc, #0xFF00	@ r0 = FFFE (IRQ vector address)
   readpc16
   ldrb r1, [r0]		@ get opcode in r1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

.ltorg	@ literal storage

#
# Jump table used by the FETCH macro to execute opcodes
#
jmp:				
	b	C_00		@ BRK_IMP
	b	C_01		@ ORA_INDX
	b	C_02		@ INV2
	b	C_03		@ INV
	b	C_04		@ TSB_ZPG
	b	C_05		@ ORA_ZPG
	b	C_06		@ ASL_ZPG
	b	C_07		@ RMB0_ZPG
	b	C_08		@ PHP_IMP
	b	C_09		@ ORA_IMM
	b	C_0A		@ ASL_IMP
	b	C_0B		@ INV
	b	C_0C		@ TSB_ABS
	b	C_0D		@ ORA_ABS
	b	C_0E		@ ASL_ABS
	b	C_0F		@ BBR0_REL
	b	C_10		@ BPL_REL
	b	C_11		@ ORA_INDY
	b	C_12		@ ORA_IND
	b	C_13		@ INV
	b	C_14		@ TRB_ZPG
	b	C_15		@ ORA_ZPGX
	b	C_16		@ ASL_ZPGX
	b	C_17		@ RMB1_ZPG
	b	C_18		@ CLC_IMP
	b	C_19		@ ORA_ABSY
	b	C_1A		@ INC_IMP
	b	C_1B		@ INV
	b	C_1C		@ TRB_ABS
	b	C_1D		@ ORA_ABSX
	b	C_1E		@ ASL_ABSX
	b	C_1F		@ BBR1_REL
	b	C_20		@ JSR_ABS
	b	C_21		@ AND_INDX
	b	C_22		@ INV2
	b	C_23		@ INV
	b	C_24		@ BIT_ZPG
	b	C_25		@ AND_ZPG
	b	C_26		@ ROL_ZPG
	b	C_27		@ RMB2_ZPG
	b	C_28		@ PLP_IMP
	b	C_29		@ AND_IMM
	b	C_2A		@ ROL_IMP
	b	C_2B		@ INV
	b	C_2C		@ BIT_ABS
	b	C_2D		@ AND_ABS
	b	C_2E		@ ROL_ABS
	b	C_2F		@ BBR2_REL
	b	C_30		@ BMI_REL
	b	C_31		@ AND_INDY
	b	C_32		@ AND_IND
	b	C_33		@ INV
	b	C_34		@ BIT_ZPGX
	b	C_35		@ AND_ZPGX
	b	C_36		@ ROL_ZPGX
	b	C_37		@ RMB3_ZPG
	b	C_38		@ SEC_IMP
	b	C_39		@ AND_ABSY
	b	C_3A		@ DEC_IMP
	b	C_3B		@ INV
	b	C_3C		@ BIT_ABSX
	b	C_3D		@ AND_ABSX
	b	C_3E		@ ROL_ABSX
	b	C_3F		@ BBR3_REL
	b	C_40		@ RTI_IMP
	b	C_41		@ EOR_INDX
	b	C_42		@ INV2
	b	C_43		@ INV
	b	C_44		@ INV2
	b	C_45		@ EOR_ZPG
	b	C_46		@ LSR_ZPG
	b	C_47		@ RMB4_ZPG
	b	C_48		@ PHA_IMP
	b	C_49		@ EOR_IMM
	b	C_4A		@ LSR_IMP
	b	C_4B		@ INV
	b	C_4C		@ JMP_ABS
	b	C_4D		@ EOR_ABS
	b	C_4E		@ LSR_ABS
	b	C_4F		@ BBR4_REL
	b	C_50		@ BVC_REL
	b	C_51		@ EOR_INDY
	b	C_52		@ EOR_IND
	b	C_53		@ INV
	b	C_54		@ INV2
	b	C_55		@ EOR_ZPGX
	b	C_56		@ LSR_ZPGX
	b	C_57		@ RMB5_ZPG
	b	C_58		@ CLI_IMP
	b	C_59		@ EOR_ABSY
	b	C_5A		@ PHY_IMP
	b	C_5B		@ INV
	b	C_5C		@ INV3
	b	C_5D		@ EOR_ABSX
	b	C_5E		@ LSR_ABSX
	b	C_5F		@ BBR5_REL
	b	C_60		@ RTS_IMP
	b	C_61		@ ADC_INDX
	b	C_62		@ INV2
	b	C_63		@ INV
	b	C_64		@ STZ_ZPG
	b	C_65		@ ADC_ZPG
	b	C_66		@ ROR_ZPG
	b	C_67		@ RMB6_ZPG
	b	C_68		@ PLA_IMP
	b	C_69		@ ADC_IMM
	b	C_6A		@ ROR_IMP
	b	C_6B		@ INV
	b	C_6C		@ JMP_JIND
	b	C_6D		@ ADC_ABS
	b	C_6E		@ ROR_ABS
	b	C_6F		@ BBR6_REL
	b	C_70		@ BVS_REL
	b	C_71		@ ADC_INDY
	b	C_72		@ ADC_IND
	b	C_73		@ INV
	b	C_74		@ STZ_ZPGX
	b	C_75		@ ADC_ZPGX
	b	C_76		@ ROR_ZPGX
	b	C_77		@ RMB7_ZPG
	b	C_78		@ SEI_IMP
	b	C_79		@ ADC_ABSY
	b	C_7A		@ PLY_IMP
	b	C_7B		@ INV
	b	C_7C		@ JMP_JINDX
	b	C_7D		@ ADC_ABSX
	b	C_7E		@ ROR_ABSX
	b	C_7F		@ BBR7_REL
	b	C_80		@ BRA_REL
	b	C_81		@ STA_INDX
	b	C_82		@ INV2
	b	C_83		@ INV
	b	C_84		@ STY_ZPG
	b	C_85		@ STA_ZPG
	b	C_86		@ STX_ZPG
	b	C_87		@ SMB0_ZPG
	b	C_88		@ DEY_IMP
	b	C_89		@ BIT_IMM
	b	C_8A		@ TXA_IMP
	b	C_8B		@ INV
	b	C_8C		@ STY_ABS
	b	C_8D		@ STA_ABS
	b	C_8E		@ STX_ABS
	b	C_8F		@ BBS0_REL
	b	C_90		@ BCC_REL
	b	C_91		@ STA_INDY
	b	C_92		@ STA_IND
	b	C_93		@ INV
	b	C_94		@ STY_ZPGX
	b	C_95		@ STA_ZPGX
	b	C_96		@ STX_ZPGY
	b	C_97		@ SMB1_ZPG
	b	C_98		@ TYA_IMP
	b	C_99		@ STA_ABSY
	b	C_9A		@ TXS_IMP
	b	C_9B		@ INV
	b	C_9C		@ STZ_ABS
	b	C_9D		@ STA_ABSX
	b	C_9E		@ STZ_ABSX
	b	C_9F		@ BBS1_REL
	b	C_A0		@ LDY_IMM
	b	C_A1		@ LDA_INDX
	b	C_A2		@ LDX_IMM
	b	C_A3		@ INV
	b	C_A4		@ LDY_ZPG
	b	C_A5		@ LDA_ZPG
	b	C_A6		@ LDX_ZPG
	b	C_A7		@ SMB2_ZPG
	b	C_A8		@ TAY_IMP
	b	C_A9		@ LDA_IMM
	b	C_AA		@ TAX_IMP
	b	C_AB		@ INV
	b	C_AC		@ LDY_ABS
	b	C_AD		@ LDA_ABS
	b	C_AE		@ LDX_ABS
	b	C_AF		@ BBS2_REL
	b	C_B0		@ BCS_REL
	b	C_B1		@ LDA_INDY
	b	C_B2		@ LDA_IND
	b	C_B3		@ INV
	b	C_B4		@ LDY_ZPGX
	b	C_B5		@ LDA_ZPGX
	b	C_B6		@ LDX_ZPGY
	b	C_B7		@ SMB3_ZPG
	b	C_B8		@ CLV_IMP
	b	C_B9		@ LDA_ABSY
	b	C_BA		@ TSX_IMP
	b	C_BB		@ INV
	b	C_BC		@ LDY_ABSX
	b	C_BD		@ LDA_ABSX
	b	C_BE		@ LDX_ABSY
	b	C_BF		@ BBS3_REL
	b	C_C0		@ CPY_IMM
	b	C_C1		@ CMP_INDX
	b	C_C2		@ INV2
	b	C_C3		@ INV
	b	C_C4		@ CPY_ZPG
	b	C_C5		@ CMP_ZPG
	b	C_C6		@ DEC_ZPG
	b	C_C7		@ SMB4_ZPG
	b	C_C8		@ INY_IMP
	b	C_C9		@ CMP_IMM
	b	C_CA		@ DEX_IMP
	b	C_CB		@ WAI_IMP
	b	C_CC		@ CPY_ABS
	b	C_CD		@ CMP_ABS
	b	C_CE		@ DEC_ABS
	b	C_CF		@ BBS4_REL
	b	C_D0		@ BNE_REL
	b	C_D1		@ CMP_INDY
	b	C_D2		@ CMP_IND
	b	C_D3		@ INV
	b	C_D4		@ INV2
	b	C_D5		@ CMP_ZPGX
	b	C_D6		@ DEC_ZPGX
	b	C_D7		@ SMB5_ZPG
	b	C_D8		@ CLD_IMP
	b	C_D9		@ CMP_ABSY
	b	C_DA		@ PHX_IMP
	b	C_DB		@ STP_IMP
	b	C_DC		@ INV3
	b	C_DD		@ CMP_ABSX
	b	C_DE		@ DEC_ABSX
	b	C_DF		@ BBS5_REL
	b	C_E0		@ CPX_IMM
	b	C_E1		@ SBC_INDX
	b	C_E2		@ INV2
	b	C_E3		@ INV
	b	C_E4		@ CPX_ZPG
	b	C_E5		@ SBC_ZPG
	b	C_E6		@ INC_ZPG
	b	C_E7		@ SMB6_ZPG
	b	C_E8		@ INX_IMP
	b	C_E9		@ SBC_IMM
	b	C_EA		@ NOP_IMP
	b	C_EB		@ INV
	b	C_EC		@ CPX_ABS
	b	C_ED		@ SBC_ABS
	b	C_EE		@ INC_ABS
	b	C_EF		@ BBS6_REL
	b	C_F0		@ BEQ_REL
	b	C_F1		@ SBC_INDY
	b	C_F2		@ SBC_IND
	b	C_F3		@ INV
	b	C_F4		@ INV2
	b	C_F5		@ SBC_ZPGX
	b	C_F6		@ INC_ZPGX
	b	C_F7		@ SMB7_ZPG
	b	C_F8		@ SED_IMP
	b	C_F9		@ SBC_ABSY
	b	C_FA		@ PLX_IMP
	b	C_FB		@ INV
	b	C_FC		@ INV3
	b	C_FD		@ SBC_ABSX
	b	C_FE		@ INC_ABSX
	b	C_FF		@ BBS7_REL

#
# Opcode commands start here - macros are used to maintain consistency, unless there is only one instance.
# See the macros.inc file for the actual assembler code.
#				
C_00:				@ BRK_IMP	DI
	C_BRK
	CYCLE 7
	FETCH	
	 			
C_01:	 			@ ORA_INDX	NZ 
	INDX			
	C_ORA			
	CYCLE 6			
	FETCH			
	 			
C_02:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_03:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_04:	 			@ TSB_ZPG	Z	
	ZPG			
	C_TSB			
	CYCLE 5			
	FETCH			
	 			
C_05:	 			@ ORA_ZPG	NZ
	ZPG			
	C_ORA			
	CYCLE 3			
	FETCH			
	 			
C_06:	 			@ ASL_ZPG	NZC
	ZPG			
	C_ASL
	CYCLE 5			
	FETCH			
	 			
C_07:	 			@ RMB0_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xFE
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_08:	 			@ PHP_IMP	-
	C_PHP			
	CYCLE 3			
	FETCH			
	 			
C_09:	 			@ ORA_IMM	NZ
	IMM			
	C_ORA			
	CYCLE 2			
	FETCH			
	 			
C_0A:	 			@ ASL_IMP	NZC
	C_ASLA			
	CYCLE 2			
	FETCH			
	 			
C_0B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_0C:	 			@ TSB_ABS	Z
	ABS			
	C_TSB			
	CYCLE 6			
	FETCH			
	 			
C_0D:	 			@ ORA_ABS	NZ
	ABS			
	C_ORA			
	CYCLE 4			
	FETCH			
	 			
C_0E:	 			@ ASL_ABS	NZC
	ABS			
	C_ASL
	CYCLE 6			
	FETCH			
	 			
C_0F:	 			@ BBR0_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #1	
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_10:	 			@ BPL_REL	-
	REL			
	C_BPL			
	CYCLE 2			
	FETCH			
	 			
C_11:	 			@ ORA_INDY	NZ
	INDY			
	C_ORA			
	CYCLE 5			
	FETCH			
	 			
C_12:	 			@ ORA_IND	NZ
	IND			
	C_ORA			
	CYCLE 5			
	FETCH			
	 			
C_13:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_14:	 			@ TRB_ZPG	Z
	ZPG			
	C_TRB			
	CYCLE 5			
	FETCH			
	 			
C_15:	 			@ ORA_ZPGX	NZ
	ZPGX			
	C_ORA			
	CYCLE 4			
	FETCH			
	 			
C_16:	 			@ ASL_ZPGX	NZC
	ZPGX			
	C_ASL
	CYCLE 6			
	FETCH			
	 			
C_17:	 			@ RMB1_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xFD
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_18:	 			@ CLC_IMP	C
	C_CLC			
	CYCLE 2			
	FETCH			
	 			
C_19:	 			@ ORA_ABSY	NZ
	ABSY			
	C_ORA			
	CYCLE 4			
	FETCH			
	 			
C_1A:	 			@ INC_IMP	NZ
	C_INCA			
	CYCLE 2			
	FETCH			
	 			
C_1B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_1C:	 			@ TRB_ABS	Z
	ABS			
	C_TRB			
	CYCLE 6			
	FETCH			
	 			
C_1D:	 			@ ORA_ABSX	NZ
	ABSX			
	C_ORA			
	CYCLE 4			
	FETCH			
	 			
C_1E:	 			@ ASL_ABSX	NZC
	ABSX			
	C_ASL
	CYCLE 6			
	FETCH			
	 			
C_1F:	 			@ BBR1_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x02	
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_20:	 			@ JSR_ABS	-
	C_JSR
	CYCLE 6			
	FETCH			
	 			
C_21:	 			@ AND_INDX	NZ
	INDX			
	C_AND			
	CYCLE 6			
	FETCH			
	 			
C_22:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_23:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_24:	 			@ BIT_ZPG	NVZ
	ZPG			
	C_BIT			
	CYCLE 3			
	FETCH			
	 			
C_25:	 			@ AND_ZPG	NZ
	ZPG			
	C_AND			
	CYCLE 3			
	FETCH			
	 			
C_26:	 			@ ROL_ZPG	NZC
	ZPG			
	C_ROL
	CYCLE 5			
	FETCH			
	 			
C_27:	 			@ RMB2_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xFB
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_28:	 			@ PLP_IMP	NV11DIZC
	C_PLP			
	CYCLE 4			
	FETCH			
	 			
C_29:	 			@ AND_IMM	NZ
	IMM			
	C_AND			
	CYCLE 2			
	FETCH			
	 			
C_2A:	 			@ ROL_IMP	NZC
	C_ROLA			
	CYCLE 2			
	FETCH			
	 			
C_2B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_2C:	 			@ BIT_ABS	NVZ
	ABS			
	C_BIT			
	CYCLE 4			
	FETCH			
	 			
C_2D:	 			@ AND_ABS	NZ
	ABS			
	C_AND			
	CYCLE 4			
	FETCH			
	 			
C_2E:	 			@ ROL_ABS	NZC
	ABS			
	C_ROL
	CYCLE 6			
	FETCH			
	 			
C_2F:	 			@ BBR2_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x04
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_30:	 			@ BMI_REL	-
	REL			
	C_BMI			
	CYCLE 2			
	FETCH			
	 			
C_31:	 			@ AND_INDY	NZ
	INDY			
	C_AND			
	CYCLE 5			
	FETCH			
	 			
C_32:	 			@ AND_IND	NZ
	IND			
	C_AND			
	CYCLE 5			
	FETCH			
	 			
C_33:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_34:	 			@ BIT_ZPGX	NVZ
	ZPGX			
	C_BIT			
	CYCLE 4			
	FETCH			
	 			
C_35:	 			@ AND_ZPGX	NZ
	ZPGX			
	C_AND			
	CYCLE 4			
	FETCH			
	 			
C_36:	 			@ ROL_ZPGX	NZC
	ZPGX			
	C_ROL
	CYCLE 6			
	FETCH			
	 			
C_37:	 			@ RMB3_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xF7
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
		
C_38:	 			@ SEC_IMP	C
	C_SEC			
	CYCLE 2			
	FETCH			
	 			
C_39:	 			@ AND_ABSY	NZ
	ABSY			
	C_AND			
	CYCLE 4			
	FETCH			
	 			
C_3A:	 			@ DEC_IMP	NZ
	C_DECA			
	CYCLE 2			
	FETCH			
	 			
C_3B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_3C:	 			@ BIT_ABSX	NVZ
	ABSX			
	C_BIT			
	CYCLE 4			
	FETCH			
	 			
C_3D:	 			@ AND_ABSX	NZ
	ABSX			
	C_AND			
	CYCLE 4			
	FETCH			
	 			
C_3E:	 			@ ROL_ABSX	NZC
	ABSX			
	C_ROL
	CYCLE 6			
	FETCH			
	 			
C_3F:	 			@ BBR3_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x08
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_40:	 			@ RTI_IMP	NV11DIZC
	C_RTI			
	CYCLE 6			
	FETCH			
	 			
C_41:	 			@ EOR_INDX	NZ
	INDX			
	C_EOR			
	CYCLE 6			
	FETCH			
	 			
C_42:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_43:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_44:	 			@ INV2
	C_INV2			
	CYCLE 3			
	FETCH			
	 			
C_45:	 			@ EOR_ZPG	NZ
	ZPG			
	C_EOR			
	CYCLE 3			
	FETCH			
	 			
C_46:	 			@ LSR_ZPG	0ZC
	ZPG			
	C_LSR			
	CYCLE 5
	FETCH			
	 			
C_47:	 			@ RMB4_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xEF
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_48:	 			@ PHA_IMP	-
	C_PHA			
	CYCLE 3			
	FETCH			
	 			
C_49:	 			@ EOR_IMM	NZ
	IMM			
	C_EOR			
	CYCLE 2			
	FETCH			
	 			
C_4A:	 			@ LSR_IMP	0ZC
	C_LSRA			
	CYCLE 2			
	FETCH			
	 			
C_4B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_4C:	 			@ JMP_ABS	-
	C_JMP			
	CYCLE 3			
	FETCH			
	 			
C_4D:	 			@ EOR_ABS	NZ
	ABS			
	C_EOR			
	CYCLE 4			
	FETCH			
	 			
C_4E:	 			@ LSR_ABS	0ZC
	ABS			
	C_LSR
	CYCLE 6			
	FETCH			
	 			
C_4F:	 			@ BBR4_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x10
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_50:	 			@ BVC_REL	-
	REL			
	C_BVC			
	CYCLE 2			
	FETCH			
	 			
C_51:	 			@ EOR_INDY	NZ
	INDY			
	C_EOR			
	CYCLE 5			
	FETCH			
	 			
C_52:	 			@ EOR_IND	NZ
	IND			
	C_EOR			
	CYCLE 5			
	FETCH			
	 			
C_53:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_54:	 			@ INV2
	C_INV2			
	CYCLE 4			
	FETCH			
	 			
C_55:	 			@ EOR_ZPGX	NZ
	ZPGX			
	C_EOR			
	CYCLE 4			
	FETCH			
	 			
C_56:	 			@ LSR_ZPGX	0ZC
	ZPGX			
	C_LSR			
	CYCLE 6			
	FETCH			
	 			
C_57:	 			@ RMB5_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xDF
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_58:	 			@ CLI_IMP	I
	C_CLI			
	CYCLE 2			
	FETCH			
	 			
C_59:	 			@ EOR_ABSY	NZ
	ABSY			
	C_EOR			
	CYCLE 4			
	FETCH			
	 			
C_5A:	 			@ PHY_IMP	-
	C_PHY			
	CYCLE 3			
	FETCH			
	 			
C_5B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_5C:	 			@ INV3
	C_INV3			
	CYCLE 8			
	FETCH			
	 			
C_5D:	 			@ EOR_ABSX	NZ
	ABSX			
	C_EOR			
	CYCLE 4			
	FETCH			
	 			
C_5E:	 			@ LSR_ABSX	0ZC	
	ABSX			
	C_LSR
	CYCLE 6			
	FETCH			
	 			
C_5F:	 			@ BBR5_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x20
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_60:	 			@ RTS_IMP	-	
	C_RTS			
	CYCLE 6			
	FETCH			
	 			
C_61:	 			@ ADC_INDX	NVZC
	INDX			
	C_ADC			
	CYCLE 6			
	FETCH			
	 			
C_62:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_63:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_64:	 			@ STZ_ZPG	-
	ZPG			
	C_STZ			
	CYCLE 3			
	FETCH			
	 			
C_65:	 			@ ADC_ZPG	NVZC
	ZPG			
	C_ADC			
	CYCLE 3			
	FETCH			
	 			
C_66:	 			@ ROR_ZPG	NZC
	ZPG			
	C_ROR
	CYCLE 5			
	FETCH			
	 			
C_67:	 			@ RMB6_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0xBF
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_68:	 			@ PLA_IMP	NZ
	C_PLA			
	CYCLE 4			
	FETCH			
	 			
C_69:	 			@ ADC_IMM	NVZC
	IMM			
	C_ADC			
	CYCLE 2			
	FETCH			
	 			
C_6A:	 			@ ROR_IMP	NZC
	C_RORA			
	CYCLE 2			
	FETCH			
	 			
C_6B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_6C:	 			@ JMP_JIND	-
	JIND			
	CYCLE 6			
	FETCH			
	 			
C_6D:	 			@ ADC_ABS	NVZC
	ABS			
	C_ADC			
	CYCLE 4			
	FETCH			
	 			
C_6E:	 			@ ROR_ABS	NZC
	ABS			
	C_ROR
	CYCLE 6			
	FETCH			
	 			
C_6F:	 			@ BBR6_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x40
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_70:	 			@ BVS_REL	-
	REL			
	C_BVS			
	CYCLE 2			
	FETCH			
	 			
C_71:	 			@ ADC_INDY	NVZC
	INDY			
	C_ADC			
	CYCLE 5			
	FETCH			
	 			
C_72:	 			@ ADC_IND	NVZC
	IND			
	C_ADC			
	CYCLE 5			
	FETCH			
	 			
C_73:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_74:	 			@ STZ_ZPGX	-
	ZPGX			
	C_STZ			
	CYCLE 4			
	FETCH			
	 			
C_75:	 			@ ADC_ZPGX	NVZC
	ZPGX			
	C_ADC			
	CYCLE 4			
	FETCH			
	 			
C_76:	 			@ ROR_ZPGX	NZC
	ZPGX			
	C_ROR			
	CYCLE 6			
	FETCH			
	 			
C_77:	 			@ RMB7_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   and r1, r1, #0x7F
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_78:	 			@ SEI_IMP	I
	C_SEI			
	CYCLE 2			
	FETCH			
	 			
C_79:	 			@ ADC_ABSY	NVZC
	ABSY			
	C_ADC			
	CYCLE 4			
	FETCH			
	 			
C_7A:	 			@ PLY_IMP	NZ
	C_PLY			
	CYCLE 4			
	FETCH			
	 			
C_7B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_7C:	 			@ JMP_JINDX	-
	JINDX			
	CYCLE 6			
	FETCH			
	 			
C_7D:	 			@ ADC_ABSX	NVZC
	ABSX			
	C_ADC			
	CYCLE 4			
	FETCH			
	 			
C_7E:	 			@ ROR_ABSX	NZC
	ABSX			
	C_ROR			
	CYCLE 6			
	FETCH			
	 			
C_7F:	 			@ BBR7_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x80
   moveq rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_80:	 			@ BRA_REL	-
	REL			
	C_BRA			
	CYCLE 3			
	FETCH			
	 			
C_81:	 			@ STA_INDX	-
	INDX			
	C_STA			
	CYCLE 6			
	FETCH			
	 			
C_82:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_83:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_84:	 			@ STY_ZPG	-
	ZPG			
	C_STY			
	CYCLE 3			
	FETCH			
	 			
C_85:	 			@ STA_ZPG	-
	ZPG			
	C_STA			
	CYCLE 3			
	FETCH			
	 			
C_86:	 			@ STX_ZPG	-
	ZPG			
	C_STX			
	CYCLE 3			
	FETCH			
	 			
C_87:	 			@ SMB0_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x01
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_88:	 			@ DEY_IMP	NZ
	C_DEY			
	CYCLE 2			
	FETCH			
	 			
C_89:	 				@ BIT_IMM	Z
	IMM				@ bit # mode only affects the Z flag, so doing code here vs. macro
	and  rsr, rsr, #0xFD		@ clear Z
   	ands r2, acc, r1, LSL #24  	@ test AND's contents of acc & mem
   	orreq rsr, rsr, #0x02		@ set Z flag
	CYCLE 2			
	FETCH			
	 			
C_8A:	 			@ TXA_IMP	NZ
	C_TXA			
	CYCLE 2			
	FETCH			
	 			
C_8B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_8C:	 			@ STY_ABS	-
	ABS			
	C_STY			
	CYCLE 4			
	FETCH			
	 			
C_8D:	 			@ STA_ABS	-
	ABS			
	C_STA			
	CYCLE 4			
	FETCH			
	 			
C_8E:	 			@ STX_ABS	-
	ABS			
	C_STX			
	CYCLE 4			
	FETCH			
	 			
C_8F:	 			@ BBS0_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x01
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_90:	 			@ BCC_REL	-
	REL			
	C_BCC			
	CYCLE 2			
	FETCH			
	 			
C_91:	 			@ STA_INDY	-
	INDY			
	C_STA			
	CYCLE 6			
	FETCH			
	 			
C_92:	 			@ STA_IND	-
	IND			
	C_STA			
	CYCLE 5			
	FETCH			
	 			
C_93:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_94:	 			@ STY_ZPGX	-
	ZPGX			
	C_STY			
	CYCLE 4			
	FETCH			
	 			
C_95:	 			@ STA_ZPGX	-
	ZPGX			
	C_STA			
	CYCLE 4			
	FETCH			
	 			
C_96:	 			@ STX_ZPGY	-
	ZPGY			
	C_STX			
	CYCLE 4			
	FETCH			
	 			
C_97:	 			@ SMB1_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x02
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_98:	 			@ TYA_IMP	NZ
	C_TYA			
	CYCLE 2			
	FETCH			
	 			
C_99:	 			@ STA_ABSY	-
	ABSY			
	C_STA			
	CYCLE 5			
	FETCH			
	 			
C_9A:	 			@ TXS_IMP	-
	C_TXS			
	CYCLE 2			
	FETCH			
	 			
C_9B:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_9C:	 			@ STZ_ABS	-
	ABS			
	C_STZ			
	CYCLE 4			
	FETCH			
	 			
C_9D:	 			@ STA_ABSX	-
	ABSX			
	C_STA			
	CYCLE 4			
	FETCH			
	 			
C_9E:	 			@ STZ_ABSX	-
	ABSX			
	C_STZ			
	CYCLE 5			
	FETCH			
	 			
C_9F:	 			@ BBS1_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x02
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_A0:	 			@ LDY_IMM	NZ
	IMM			
	C_LDY			
	CYCLE 2			
	FETCH			
	 			
C_A1:	 			@ LDA_INDX	NZ
	INDX			
	C_LDA			
	CYCLE 6			
	FETCH			
	 			
C_A2:	 			@ LDX_IMM	NZ
	IMM			
	C_LDX			
	CYCLE 2			
	FETCH			
	 			
C_A3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_A4:	 			@ LDY_ZPG	NZ
	ZPG			
	C_LDY			
	CYCLE 3			
	FETCH			
	 			
C_A5:	 			@ LDA_ZPG	NZ
	ZPG			
	C_LDA			
	CYCLE 3			
	FETCH			
	 			
C_A6:	 			@ LDX_ZPG	NZ
	ZPG			
	C_LDX			
	CYCLE 3			
	FETCH			
	 			
C_A7:	 			@ SMB2_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x04
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_A8:	 			@ TAY_IMP	NZ
	C_TAY			
	CYCLE 2			
	FETCH			
	 			
C_A9:	 			@ LDA_IMM	NZ
	IMM			
	C_LDA			
	CYCLE 2			
	FETCH			
	 			
C_AA:	 			@ TAX_IMP	NZ
	C_TAX			
	CYCLE 2			
	FETCH			
	 			
C_AB:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_AC:	 			@ LDY_ABS	NZ
	ABS			
	C_LDY			
	CYCLE 4			
	FETCH			
	 			
C_AD:	 			@ LDA_ABS	NZ
	ABS			
	C_LDA			
	CYCLE 4			
	FETCH			
	 			
C_AE:	 			@ LDX_ABS	NZ
	ABS			
	C_LDX			
	CYCLE 4			
	FETCH			
	 			
C_AF:	 			@ BBS2_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x04
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_B0:	 			@ BCS_REL	-
	REL			
	C_BCS			
	CYCLE 2			
	FETCH			
	 			
C_B1:	 			@ LDA_INDY	NZ
	INDY			
	C_LDA			
	CYCLE 5			
	FETCH			
	 			
C_B2:	 			@ LDA_IND	NZ
	IND			
	C_LDA			
	CYCLE 5		
	FETCH			
	 			
C_B3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_B4:	 			@ LDY_ZPGX	NZ
	ZPGX			
	C_LDY			
	CYCLE 4			
	FETCH			
	 			
C_B5:	 			@ LDA_ZPGX	NZ
	ZPGX			
	C_LDA			
	CYCLE 4			
	FETCH			
	 			
C_B6:	 			@ LDX_ZPGY	NZ
	ZPGY			
	C_LDX			
	CYCLE 4			
	FETCH			
	 			
C_B7:	 			@ SMB3_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x08
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_B8:	 			@ CLV_IMP	V
	C_CLV			
	CYCLE 2			
	FETCH			
	 			
C_B9:	 			@ LDA_ABSY	NZ
	ABSY			
	C_LDA			
	CYCLE 4			
	FETCH			
	 			
C_BA:	 			@ TSX_IMP	NZ
	C_TSX			
	CYCLE 2			
	FETCH			
	 			
C_BB:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_BC:	 			@ LDY_ABSX	NZ
	ABSX			
	C_LDY			
	CYCLE 4			
	FETCH			
	 			
C_BD:	 			@ LDA_ABSX	NZ
	ABSX			
	C_LDA			
	CYCLE 4			
	FETCH			
	 			
C_BE:	 			@ LDX_ABSY	NZ
	ABSY			
	C_LDX			
	CYCLE 4			
	FETCH			
	 			
C_BF:	 			@ BBS3_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x08
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_C0:	 			@ CPY_IMM	NZC
	IMM			
	C_CPY			
	CYCLE 2			
	FETCH			
	 			
C_C1:	 			@ CMP_INDX	NZC
	INDX			
	C_CMP			
	CYCLE 6			
	FETCH			
	 			
C_C2:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_C3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_C4:	 			@ CPY_ZPG	NZC
	ZPG			
	C_CPY			
	CYCLE 3			
	FETCH			
	 			
C_C5:	 			@ CMP_ZPG	NZC
	ZPG			
	C_CMP			
	CYCLE 3			
	FETCH			
	 			
C_C6:	 			@ DEC_ZPG	NZ
	ZPG			
	C_DEC			
	CYCLE 5			
	FETCH			
	 			
C_C7:	 			@ SMB4_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x10
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_C8:	 			@ INY_IMP	NZ
	C_INY			
	CYCLE 2			
	FETCH			
	 			
C_C9:	 			@ CMP_IMM	NZC
	IMM			
	C_CMP			
	CYCLE 2			
	FETCH			
	 			
C_CA:	 			@ DEX_IMP	NZ
	C_DEX			
	CYCLE 2			
	FETCH			
	 			
C_CB:	 			@ WAI_IMP	-
	C_WAI			
	CYCLE 3			
	FETCH			
	 			
C_CC:	 			@ CPY_ABS	NZC
	ABS			
	C_CPY			
	CYCLE 4			
	FETCH			
	 			
C_CD:	 			@ CMP_ABS	NZC
	ABS			
	C_CMP			
	CYCLE 4			
	FETCH			
	 			
C_CE:	 			@ DEC_ABS	NZ
	ABS			
	C_DEC			
	CYCLE 6			
	FETCH			
	 			
C_CF:	 			@ BBS4_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x10
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_D0:	 			@ BNE_REL	-
	REL			
	C_BNE			
	CYCLE 2			
	FETCH			
	 			
C_D1:	 			@ CMP_INDY	NZC
	INDY			
	C_CMP			
	CYCLE 5			
	FETCH			
	 			
C_D2:	 			@ CMP_IND	NZC
	IND			
	C_CMP			
	CYCLE 5			
	FETCH			
	 			
C_D3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_D4:	 			@ INV2
	C_INV2			
	CYCLE 4			
	FETCH			
	 			
C_D5:	 			@ CMP_ZPGX	NZC
	ZPGX			
	C_CMP			
	CYCLE 4			
	FETCH			
	 			
C_D6:	 			@ DEC_ZPGX	NZ
	ZPGX			
	C_DEC			
	CYCLE 6			
	FETCH			
	 			
C_D7:	 			@ SMB5_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x20
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_D8:	 			@ CLD_IMP	D
	C_CLD			
	CYCLE 2			
	FETCH			
	 			
C_D9:	 			@ CMP_ABSY	NZC
	ABSY			
	C_CMP			
	CYCLE 4			
	FETCH			
	 			
C_DA:	 			@ PHX_IMP	-
	C_PHX			
	CYCLE 3			
	FETCH			
	 			
C_DB:	 			@ STP_IMP	-
	C_STP			
	CYCLE 3			
	FETCH			
	 			
C_DC:	 			@ INV3
	C_INV3			
	CYCLE 4			
	FETCH			
	 			
C_DD:	 			@ CMP_ABSX	NZC
	ABSX			
	C_CMP			
	CYCLE 4			
	FETCH			
	 			
C_DE:	 			@ DEC_ABSX	NZ
	ABSX			
	C_DEC			
	CYCLE 6			
	FETCH			
	 			
C_DF:	 			@ BBS5_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x20
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_E0:	 			@ CPX_IMM	NZC
	IMM			
	C_CPX			
	CYCLE 2			
	FETCH			
	 			
C_E1:	 			@ SBC_INDX	NVZC
	INDX			
	C_SBC			
	CYCLE 6			
	FETCH			
	 			
C_E2:	 			@ INV2
	C_INV2			
	CYCLE 2			
	FETCH			
	 			
C_E3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_E4:	 			@ CPX_ZPG	NZC
	ZPG			
	C_CPX			
	CYCLE 3			
	FETCH			
	 			
C_E5:	 			@ SBC_ZPG	NVZC
	ZPG			
	C_SBC			
	CYCLE 3			
	FETCH			
	 			
C_E6:	 			@ INC_ZPG	NZ
	ZPG			
	C_INC			
	CYCLE 5			
	FETCH			
	 			
C_E7:	 			@ SMB6_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x40
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_E8:	 			@ INX_IMP	NZ
	C_INX			
	CYCLE 2			
	FETCH			
	 			
C_E9:	 			@ SBC_IMM	NVZC
	IMM			
	C_SBC			
	CYCLE 2			
	FETCH			
	 			
C_EA:	 			@ NOP_IMP	-
	C_NOP			
	CYCLE 2			
	FETCH			
	 			
C_EB:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_EC:	 			@ CPX_ABS	NZC
	ABS			
	C_CPX			
	CYCLE 4			
	FETCH			
	 			
C_ED:	 			@ SBC_ABS	NVZC
	ABS			
	C_SBC			
	CYCLE 4			
	FETCH			
	 			
C_EE:	 			@ INC_ABS	NZ
	ABS			
	C_INC			
	CYCLE 6			
	FETCH			
	 			
C_EF:	 			@ BBS6_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x40
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_F0:	 			@ BEQ_REL	-
	REL			
	C_BEQ			
	CYCLE 2			
	FETCH			
	 			
C_F1:	 			@ SBC_INDY	NVZC
	INDY			
	C_SBC			
	CYCLE 5			
	FETCH			
	 			
C_F2:	 			@ SBC_IND	NVZC
	IND			
	C_SBC			
	CYCLE 5			
	FETCH			
	 			
C_F3:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_F4:	 			@ INV2
	C_INV2			
	CYCLE 4			
	FETCH			
	 			
C_F5:	 			@ SBC_ZPGX	NVZC
	ZPGX			
	C_SBC			
	CYCLE 4			
	FETCH			
	 			
C_F6:	 			@ INC_ZPGX	NZ
	ZPGX			
	C_INC			
	CYCLE 6			
	FETCH			
	 			
C_F7:	 			@ SMB7_ZPG	-
   ldrb r0, [rpc], #1
   ldrb r1, [r0]		@ load data byte
   orr r1, r1, #0x80
   strb r1, [r0]
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
	 			
C_F8:	 			@ SED_IMP	D
	C_SED			
	CYCLE 2			
	FETCH			
	 			
C_F9:	 			@ SBC_ABSY	NVZC
	ABSY			
	C_SBC			
	CYCLE 4			
	FETCH			
	 			
C_FA:	 			@ PLX_IMP	NZ
	C_PLX			
	CYCLE 4			
	FETCH			
	 			
C_FB:	 			@ INV
	C_INV			
	CYCLE 1			
	FETCH			
	 			
C_FC:	 			@ INV3
	C_INV3			
	CYCLE 4			
	FETCH			
	 			
C_FD:	 			@ SBC_ABSX	NVZC
	ABSX			
	C_SBC			
	CYCLE 4			
	FETCH			
	 			
C_FE:	 			@ INC_ABSX	NZ
	ABSX			
	C_INC			
	CYCLE 6			
	FETCH			
	 			
C_FF:	 			@ BBS7_REL	-
   ldm rpc!, {r0, r1} 
   ldrb r0, [r0]		@ load data byte
   sxtab r1, rpc, r1 
   mov r1, r1, ROR #16
   mov r1, r1, LSR #16		@ AND #0xFFFF
   ands r0, r0, #0x80
   movne rpc, r1
   @ test for NMI, IRQ, WAI	
   ldrb r1, [rpc], #1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
#
# End
#
