#################################################################################
#                                                                               #
#  65C02 Simulator for PI  v1.0                             Feb 23, 2018        #
#                                                                               #
#  Written by Daryl Ritor with credit to David Welch's awesome tutorials        #
#                                                                               #
#################################################################################
#
# Memory and IO access 
#
.macro readpc			@ r1 = (rpc)  
   ldrb r1, [rpc], #1
.endm

.macro readpc16			@ r0 = (rpc + 256*(rpc+1))
@   ldrh r0, [rpc], #2		@ this method is faster, but does not check 0x00FF or 0x7FFF
   readpc			@ read byte from RAM or ROM only
   mov r2, r1			@ save lower byte
   readpc			@ read byte from RAM or ROM only	
   ORR r0,r2,r1, ASL #8		@ combine 2 bytes
.endm

.macro readaddr 		@ r1 = (r0)
   cmp iobase, r0, LSR #8
   ldrneb r1, [r0]
   bleq readio   
.endm

.macro readaddr16		@ r1 = (r0)
@   ldrh r1, [r0]		@ this method is faster, but does not check 0x00FF or 0x7FFF
   readaddr			@ read byte from RAM, ROM, or IO
   mov r2, r1			@ save lower byte
   add r0, r0, #1		@ inc address
   readaddr			@ read byte from RAM, ROM, or IO
   ORR r1,r2,r1, ASL #8		@ combine 2 bytes
   sub r0, r0, #1		@ reset pointer (for writes)
.endm

.macro writeaddr		@ source=r0, data=r1
   cmp iobase, r0, LSR #8
   strplb r1, [r0]		@ if making all memory RAM, change this to 'strneb r1, [r0]'
   bleq writeio   
.endm

#
# Stack access
#
.macro pushs			@ data=r0
   strb r0, [rsp, #0x100]	@ push to stack
   sub rsp, rsp, #1		@ dec pointer
   and rsp, rsp, #0xFF		@ mask to 1 byte
.endm

.macro pushs16			@ data=r0
   mov r1, r0, lsr #8		@ hi byte first
   strb r1, [rsp, #0x100]	@ push to stack
   sub rsp, rsp, #1		@ dec pointer
   and rsp, rsp, #0xFF		@ mask to 1byte
   strb r0, [rsp, #0x100]	@ push lower byte to stack
   sub rsp, rsp, #1		@ dec pointer
   and rsp, rsp, #0xFF		@ mask to 1 byte
.endm

.macro pops			@ data=r0
   add rsp, rsp, #1		@ pre-inc pointer
   and rsp, rsp, #0xFF		@ mask to byte
   ldrb r0, [rsp, #0x100]	@ pull from stack
.endm

.macro pops16			@ data=r0
   add rsp, rsp, #1		@ pre-inc pointer
   and rsp, rsp, #0xFF		@ mask to 1 byte
   ldrb r0, [rsp, #0x100]	@ pull from stack low byte
   add rsp, rsp, #1		@ inc pointer
   and rsp, rsp, #0xFF		@ mask to 1 byte
   ldrb r1, [rsp, #0x100]	@ pull from stack hi byte
   ORR r0,r0,r1,ASL #8		@ combine two bytes
.endm

#
# Processor control
#

.macro CYCLE a
  @ add cyc, cyc, #\a - not using cycle counting
.endm
.macro ECYCLE a
  @ addeq cyc, cyc, #\a
.endm
.macro NCYCLE a
  @ addne cyc, cyc, #\a
.endm

.macro RESET
   mov rpc, #0xFC		@ PC = 0xFC
   orr rpc, rpc, #0xFF00	@ PC = 0xFFFC
   readpc16			@ get word at $FFFC
   mov rpc, r0			@ set PC 
   mov acc, #0			@ Acc =0
   mov x, #0			@ X=0
   mov y, #0			@ Y=0
   mov rsp, #0xFF		@ SP = 0xFF
   mov rsr, #0x34		@ SR = 0x34
   mov int, #0			@ clear IRQ, NMI, STP, WAI
   @mov cyc, #0			@ cycle counter = 0
.endm

.macro FETCH
   readpc			@ get opcode in r1
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor
.endm

#
# Address Modes
#
.macro ZPG
   ldrb r0, [rpc], #1		@ read address from RAM	and inc PC
   ldrb r1, [r0]		@ load data byte
.endm

.macro ZPGX
   ldrb r0, [rpc], #1		@ read address from RAM	and inc PC
   add r0, r0, x, LSR #24	@ add X register 
   and r0, r0, #0xFF		@ mask to zero page
   ldrb r1, [r0]		@ load data byte
.endm

.macro ZPGY
   ldrb r0, [rpc], #1		@ read address from RAM	and inc PC
   add r0, r0, y, LSR #24	@ add Y register
   and r0, r0, #0xFF		@ mask to zero page
   ldrb r1, [r0]		@ load data byte
.endm

.macro IND			@ don't need to save address
   ldrb r1, [rpc], #1		@ read address from RAM	and inc PC
   ldrb r0, [r1], #1		@ load low byte and inc pointer
   and  r1, r1, #0xFF		@ mask address to zero page
   ldrb r1, [r1]		@ load high byte
   ORR r0, r0, r1, ASL #8	@ combine 2 bytes into target address
   readaddr			@ read from target address
.endm

.macro INDX			@ don't need to save address
   ldrb r1, [rpc], #1		@ read address from RAM	and inc PC
   add r1, r1, x, LSR #24	@ add X register
   and r1, r1, #0xFF		@ mask to zero page
   ldrb r0, [r1], #1		@ load low byte and inc pointer
   and  r1, r1, #0xFF		@ mask to zero page
   ldrb r1, [r1]		@ load high byte
   ORR r0, r0, r1, ASL #8	@ combine 2 byte for targe address
   readaddr			@ read from target address
.endm

.macro INDY			@ don't need to save address
   ldrb r1, [rpc], #1		@ read address from RAM	and inc PC	
   ldrb r0, [r1], #1		@ load low byte and inc pointer
   and  r1, r1, #0xFF		@ mask o zero page
   ldrb r1, [r1]		@ load high byte
   ORR r0, r0, r1, ASL #8	@ combine 2 bytes into target
   add r0, r0, y, LSR #24	@ add Y register to target
   readaddr			@ read from target address
.endm

.macro IMM
   readpc			@ read address from RAM	and inc PC
.endm

.macro REL			@ address in r1
   readpc			@ read address from RAM	and inc PC
   sxtab r1, rpc, r1 		@ do signed byte addition to calc new PC
   mov r1, r1, ROR #16		@ move to upper half-word
   mov r1, r1, LSR #16		@ LSR is like AND #0x0000FFFF
.endm

.macro ABS
   readpc16			@ get 16 bit address and inc PC
   readaddr			@ read from that address
.endm

.macro ABSX
   readpc16			@ get 16 bit address and inc PC
   add r0, r0, x, LSR #24 	@ add X register
   readaddr			@ read from that address
.endm

.macro ABSY
   readpc16			@ get 16 bit address and inc PC	
   add r0, r0, y, LSR #24 	@ add Y register
   readaddr			@ read from that address
.endm

.macro JIND
   readpc16			@ get 16 bit address and inc PC	
   readaddr16			@ get 16 bit value from that address
   mov rpc, r1			@ set new PC
.endm

.macro JINDX
   readpc16			@ get 16 bit address and inc PC		
   add r0, r0, x, LSR #24	@ add X register
   readaddr16			@ get 16 bit value from that address
   mov rpc, r1			@ set new PC
.endm

#
# Opcodes
#

C_ADCB:  @ (no macro) 
   sbc acc, acc, #0xFF000000	@ r2 = acc + 0x00FFFFFF (Carry=0 on entry)
   tst rsr, rsr, ROR #1		@ mov C into Carry
   and  rsr, rsr, #0x3C		@ clear N,V,Z,C
   adcs acc, acc, r1, LSL #24	@ add two values
   orrvs rsr, rsr, #0x40	@ set V flag
   orrmi rsr, rsr, #0x80	@ set N flag
   orrcs rsr, rsr, #0x01	@ set C flag
   ands acc, acc, #0xFF000000	@ mask Acc to top 8 bits
   orreq rsr, rsr, #0x02	@ set Z flag
   ldrb r1, [rpc], #1		@ fetch next opcode
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor


C_ADCD:  @ (no macro) 
   tst rsr, rsr, ROR #1		@ mov C into Carry
   and  rsr, rsr, #0x7C		@ clear N,Z,C
   and r0, acc, #0x0F000000	@ mask lower nibble
   and r2, r1, #0x0F		@ mask lower nibble
   adc r0, r2, r0, LSR #24	@ do binary add with carry
   cmp r0, #0x0A		@ is it 10 or more?
   addpl r0, r0, #0x06		@ yes, adjust it up
   mov acc, acc, LSR#28		@ mask upper nibble
   mov r1, r1, LSR#4		@ mask upper nibble
   tst r0, r0, ROR #5		@ set half carry in Carry
   and r0, r0, #0x0F		@ mask lower nibble
   adc acc, acc, r1		@ binary add with carry
   cmp acc, #0x0A		@ is it 10 or more?
   addpl acc, acc, #0x06	@ yes, adjust it up
   tst acc, acc, ROR #5		@ set carry
   orrcs rsr, rsr, #0x01	@ set C flag
   mov acc, acc, LSL #28	@ shift back to upper nibble
   orrs acc, acc, r0, LSL #24	@ combine nibbles
   orreq rsr, rsr, #0x02	@ set Z flag 
   orrmi rsr, rsr, #0x80	@ set N flag
   ldrb r1, [rpc], #1		@ fetch next opcode
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

.macro C_ADC
   tst rsr, rsr, ROR #4		@ D flag -> Carry
   bcc C_ADCB			@ do Binary ADC
   bcs C_ADCD			@ do decimal ADC
.endm

.macro C_AND
   and  rsr, rsr, #0x7D		@ clear N & Z
   ands acc, acc, r1, LSL #24	@ AND acc - R1
   orreq rsr, rsr, #02		@ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_ASL 		@ R-M-R
   and  rsr, rsr, #0x7C		@ clear N,Z & C
   movs r1, r1, LSL #25		@ do ASL
   orrcs rsr, rsr, #0x01	@ set C flag
   orreq rsr, rsr, #0x02	@ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
   mov r1, r1, LSR #24		@ 
   writeaddr
.endm

.macro C_ASLA
   and  rsr, rsr, #0x7C		@ clear N,Z & C
   movs acc, acc, LSL #1	@ do ASL
   orrcs rsr, rsr, #0x01	@ set C flag
   orreq rsr, rsr, #0x02	@ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_BBR
  @ see cpu.s   	
.endm

.macro C_BBS
  @ see cpu.s
.endm

.macro C_BCC
   ands r2, rsr, #0x01		@ mask C bit
   moveq rpc, r1		@ if 0, set newPC
   ECYCLE 1			@ add cycle for branch taken
.endm

.macro C_BCS
   ands r2, rsr, #0x01		@ mask C bit
   movne rpc, r1		@ if 1 set new PC
   NCYCLE 1			@ add cycle for branch taken
.endm

.macro C_BEQ
   ands r2, rsr, #0x02		@ mask Z bit
   movne rpc, r1		@ if 1 set new PC
   NCYCLE 1			@ add cycle for branch taken
.endm

.macro C_BIT
   and  rsr, rsr, #0x3D		@ clear N,V,Z
   and  r2, r1, #0xC0		@ mask data bits 6&7
   orr  rsr, rsr, r2		@ set SR bits 6&7
   ands r2, acc, r1, LSL #24	@ test AND's contents of acc & mem
   orreq rsr, rsr, #0x02	@ set Z flag
.endm

.macro C_BMI
   ands r2, rsr, #0x80		@ mask N bit
   movne rpc, r1		@ if 1 set new PC
   NCYCLE 1			@ add cycle for branch taken
.endm

.macro C_BNE
   ands r2, rsr, #0x02		@ mask Z bit
   moveq rpc, r1		@ if 0 set new PC
   ECYCLE 1			@ add cycle for branch taken
.endm

.macro C_BPL
   ands r2, rsr, #0x80		@ mask N bit
   moveq rpc, r1		@ if 0 set new PC
   ECYCLE 1			@ add cycle for branch taken
.endm

.macro C_BRA
   mov rpc, r1			@ set new PC always
.endm

.macro C_BRK
   add r0, rpc, #0x01		@ inc PC (skip byte after BRK opcode)
   pushs16			@ return address 
   orr r0, rsr, #0x30		@ set B bit
   pushs			@ save status w/ break bit set	
   mov r0, #0xFE		@ load IRQ address vector
   orr r0, r0, #0xFF00		@ 0xFFFE
   readaddr16			@ read target address
   and rsr, rsr, #0xF7		@ CLD
   orr rsr, rsr, #0x04     	@ SEI
   mov rpc, r1			@ set new PC
.endm

.macro C_BVC
   ands r2, rsr, #0x40		@ Mask V bit
   moveq rpc, r1		@ if 0 set new PC
   ECYCLE 1			@ add cycle for branch taken
.endm

.macro C_BVS
   ands r2, rsr, #0x40		@ mask V bit
   movne rpc, r1		@ if 1 set new PC
   NCYCLE 1			@ add cycle for branch taken
.endm

.macro C_CLC
   and rsr, rsr, #0xFE		@ clear C flag
.endm

.macro C_CLD
   and rsr, rsr, #0xF7		@ Clear D flag
.endm

.macro C_CLI
   and rsr, rsr, #0xFB		@ clear I flag
.endm

.macro C_CLV
   and rsr, rsr, #0xBF		@ clear V flag
.endm

.macro C_CMP
   and  rsr, rsr, #0x7C		@ clear N,Z,C
   cmp acc, r1, LSL #24		@ do compare
   orreq rsr, rsr, #0x02        @ set Z flag
   orrcs rsr, rsr, #0x01        @ set C flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_CPX
   and  rsr, rsr, #0x7C		@ clear N,Z,C
   cmp  x, r1, LSL #24		@ do subtraction
   orreq rsr, rsr, #0x02        @ set Z flag
   orrcs rsr, rsr, #0x01        @ set C flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_CPY
   and  rsr, rsr, #0x7C		@ clear N,Z,C
   cmp  y, r1, LSL #24		@ do subtraction
   orreq rsr, rsr, #0x02        @ set Z flag
   orrcs rsr, rsr, #0x01        @ set C flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_DEC 		@ R-M-R
   and rsr, rsr, #0x7D		@ clear N,Z
   subs r1, r1, #1		@ dec
   orreq rsr, rsr, #0x02        @ set Z flag
   and r2, r1, #0x80		@ test bit 7
   orr rsr, rsr, r2  		@ set n flag
   writeaddr			@ will write byte, no need to mask
.endm

.macro C_DECA
   and rsr, rsr, #0x7D		@ clear N,Z
   subs acc, acc, #0x1000000	@ dec
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_DEX
   and rsr, rsr, #0x7D		@ clear N,Z
   subs x, x, #0x1000000	@ dec	
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_DEY
   and rsr, rsr, #0x7D		@ clear N,Z
   subs y, y, #0x1000000	@ dec
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_EOR
   and rsr, rsr, #0x7D		@ clear N,Z
   eors acc, acc, r1, LSL #24	@ EOR
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_INC 		@ R-M-R
   and rsr, rsr, #0x7D		@ clear N,Z
   mov r1, r1, LSL #24		@ adjust to upper byte
   adds r1, r1, #0x1000000	@ inc
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
   mov r1, r1, LSR #24   	@ adjust back to lower byte
   writeaddr
.endm

.macro C_INCA
   and rsr, rsr, #0x7D		@ clear N,Z
   adds acc, acc, #0x1000000	@ inc Acc
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag.endm
.endm

.macro C_INV			@ 1 byte invalid opcode
.endm

.macro C_INV2
   add rpc, rpc, #1		@ 2 byte invalid opcode
.endm

.macro C_INV3
   add rpc, rpc, #2		@ 3 byte invalid opcode
.endm

.macro C_INX
   and rsr, rsr, #0x7D		@ clear N,Z
   adds x, x, #0x1000000	@ inc X
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_INY
   and rsr, rsr, #0x7D		@ clear N,Z
   adds y, y, #0x1000000	@ inc Y
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_JMP
   readpc16			@ get 16 bit value from PC (following opcode)
   mov rpc, r0			@ set new PC
.endm

.macro C_JSR
   add r0, rpc, #01		@ set return address-1
   pushs16			@ save to stack
   readpc16			@ get subroutine address
   mov rpc, r0			@ set PC
.endm

.macro C_LDA
   and rsr, rsr, #0x7D		@ clear N,Z
   movs acc, r1, LSL #24	@ move to Acc, shifting to upper byte
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_LDX
   and rsr, rsr, #0x7D		@ clear N,Z
   movs x, r1, LSL #24		@ move to X, shifting to upper byte
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_LDY
   and rsr, rsr, #0x7D		@ clear N,Z
   movs y, r1, LSL #24		@ move to Y, shifting to upper byte
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_LSR 		@ R-M-R
   and rsr, rsr, #0x7C		@ clear N, Z, C
   movs r1, r1, LSR #1		@ shift right with carry
   orrcs rsr, rsr, #0x01	@ set C flag
   orreq rsr, rsr, #0x02	@ set Z flag, bit 7=0, n=0 always	
   writeaddr			@ write back
.endm

.macro C_LSRA
   and rsr, rsr, #0x7C		@ clear N, Z, C
   movs acc, acc, LSR #25	@ shift right with carry
   mov acc, acc, LSL #24 	@ adjust back
   orrcs rsr, rsr, #0x01	@ set C flag
   orreq rsr, rsr, #0x02	@ set Z flag, bit 7=0, n=0 always	
.endm

.macro C_NOP
.endm

.macro C_ORA
   and  rsr, rsr, #0x7D		@ clear N & Z
   orrs acc, acc, r1, LSL #24	@ ORA
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_PHA
  mov r0, acc, LSR #24		@ shift Acc to lower byte
  pushs				@ send to stack
.endm

.macro C_PHP
   mov r0, rsr			@ get SR
   orr r0, r0, #0x30		@ set X,B flags
   pushs			@ send to stack
.endm

.macro C_PHX
  mov r0, x, LSR #24		@ shift X to lower byte
  pushs				@ send to stack
.endm

.macro C_PHY
  mov r0, y, LSR #24		@ shift Y to lower byte
  pushs				@ send to stack
.endm

.macro C_PLA
   and rsr, rsr, #0x7D		@ clear N,Z
   pops				@ get byte from stack
   movs acc, r0, LSL #24	@ move to Acc with shift to upper byte
   orreq rsr, rsr, #0x02        @ set Z	flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_PLP
   pops				@ get SR from stack
   mov rsr, r0			@ move to SR
   orr rsr, rsr, #0x30		@ set X,B flags
.endm

.macro C_PLX
   and rsr, rsr, #0x7D		@ clear N,Z
   pops				@ get X from stack
   movs x, r0, LSL #24		@ move to X with shift to upper byte
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_PLY
   and rsr, rsr, #0x7D		@ clear N,Z
   pops                         @ get Y from stack
   movs y, r0, LSL #24		@ Move to Y with shift to upper byte
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_RMB
   @ see cpu.s
.endm

.macro C_ROL 		@ R-M-R
   movs r2, rsr, LSR #1		@ move C into Carry
   and rsr, rsr, #0x7C		@ clear N,Z,C
   mov r1, r1, LSL #24		@ adjust to upper byte
   addcs r1, r1, #0x00800000	@ set carry bit in 24th bit
   movs r1, r1, LSL #1		@ do shift left with carry
   orrcs rsr, rsr, #0x01        @ set C flag
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
   mov r1, r1, LSR #24		@ shift back to lower byte
   writeaddr			@ write back
.endm

.macro C_ROLA
   movs r2, rsr, LSR #1		@ move C into Carry
   and rsr, rsr, #0x7C		@ clear N,Z,C
   addcs acc, acc, #0x00800000	@ set carry flag to 24th bit
   movs acc, acc, LSL #1	@ shift left with carry	
   orrcs rsr, rsr, #0x01        @ set C flag
   orreq rsr, rsr, #0x02        @ set Z	flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_ROR 		@ R-M-R
   movs r2, rsr, LSR #1		@ move C into Carry
   and rsr, rsr, #0x7C		@ clear N,Z,C
   orrcs rsr, rsr, #0x80	@ Set N flag
   addcs r1, r1, #0x100		@ set C flag to 9th bit 
   movs r1, r1, LSR #1   	@ shift Right with carry
   orrcs rsr, rsr, #0x01        @ set C flag
   orreq rsr, rsr, #0x02        @ set Z flag
   writeaddr
.endm

.macro C_RORA
   movs r2, rsr, LSR #1		@ move C into Carry
   and rsr, rsr, #0x7C		@ clear N,Z,C
   orrcs rsr, rsr, #0x80	@ Set N flag from Carry bit
   mov acc, acc, RRX		@ shift right with carry
   movs acc, acc, LSR #24	@ move to lower byte with carry
   orrcs rsr, rsr, #0x01        @ set C flag
   orreq rsr, rsr, #0x02        @ set Z flag
   mov acc, acc, LSL #24	@ move back to upper byte	
.endm

.macro C_RTI
   C_PLP			@ pull status register from stack
   pops16			@ pull return address from stack
   mov rpc, r0			@ set PC
.endm

.macro C_RTS
   pops16			@ pull return address from stack
   add r0, r0, #1		@ inc address
   mov rpc, r0			@ set PC
.endm

C_SBCB: @ (mo macro)
   tst rsr, rsr, ROR #1		@ mov C into Carry
   and  rsr, rsr, #0x3C		@ clear N,V,Z,C
   sbcs acc, acc, r1, lsl #24	@ SBC
   orrcs rsr, rsr, #0x01	@ set C flag
   orrmi rsr, rsr, #0x80	@ set N flag
   orrvs rsr, rsr, #0x40	@ set V flag
   ands acc, acc, #0xFF000000	@ mask to upper byte
   orreq rsr, rsr, #0x02	@ set Z flag
   ldrb r1, [rpc], #1		@ fetch next opcode
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

C_SBCD: @ (mo macro)
   tst rsr, rsr, ROR #1		@ mov C into Carry
   and rsr, rsr, #0x7C		@ clear N,Z,C
   and r0, acc, #0x0F000000	@ mask lower nibble
   and r2, r1, #0x0F		@ mask lower nibble
   sbcs r0, r0, r2, LSL #24	@ do binary subtraction with carry
   subcc r0, r0, #0x06000000	@ adjust lower nibble if borrow
   and r0, r0, #0x0F000000	@ mask lower nibble
   mov acc, acc, LSR#28		@ mask upper nibble
   mov r1, r1, LSR#4		@ mask upper nibble
   sbcs acc, acc, r1		@ binary subtraction with carry
   orrcs rsr, rsr, #0x01	@ set C flag
   subcc acc, acc, #0x06	@ adjust upper nibble if borrow
   orrs acc, r0, acc, LSL #28	@ combine nibbles
   orrmi rsr, rsr, #0x80	@ set N flag
   orreq rsr, rsr, #0x02	@ set Z flag
   ldrb r1, [rpc], #1		@ fetch next opcode
   add pc, jbase, r1, ASL #2	@ add opcode*4 to base - force jump to opcode processor

.macro C_SBC
   tst rsr, #0x08		@ mask D
   beq C_SBCB			@ do binary SBC
   bne C_SBCD			@ do decimal SBC
.endm

.macro C_SEC
   orr rsr, rsr, #0x01		@ set C flag
.endm

.macro C_SED
   orr rsr, rsr, #0x08		@ set D flag
.endm

.macro C_SEI
   orr rsr, rsr, #0x04		@ set I flag
.endm

.macro C_SMB
   @ set cpu.s	
.endm

.macro C_STA
   mov r1, acc, LSR #24		@ shift Acc to lower byte
   writeaddr			@ write to target
.endm

.macro C_STP
   ldr r0, =#0x200000		@@ bootloader backdoor address
   mov pc, r0			@ set ARM pc to bootloader
   done:  b done		@ endless loop until reset (safety net)
.endm

.macro C_STX
   mov r1, x, LSR #24 		@ shift X to lower byte	
   writeaddr			@ write to target
.endm

.macro C_STY
   mov r1, y, LSR #24		@ shift Y to lower byte
   writeaddr			@ write to target
.endm

.macro C_STZ
   mov r1, #0x00		@ set 0
   writeaddr			@ write to target
.endm

.macro C_TAX
   and rsr, rsr, #0x7D		@ clear N,Z
   movs  x, acc			@ move Acc to X
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_TAY
   and rsr, rsr, #0x7D		@ clear N,Z
   movs  y, acc			@ move Acc to Y
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_TRB
   and rsr, rsr, #0xFD		@ Z bit = 0
   ands r2, acc, r1, LSL #24	@ AND with memory
   orreq rsr, rsr, #02		@ set Z flag
   bic r1, r1, acc, LSR #24	@ reset bits masked by Acc register
   writeaddr 			@ write back
.endm

.macro C_TSB
   and rsr, rsr, #0xFD		@ Z bit = 0
   ands r2, acc, r1, LSL #24	@ AND with memory
   orreq rsr, rsr, #02		@ set Z flag
   orr r1, r1, acc, LSR #24	@ set bits masked by Acc register	
   writeaddr 			@ write back
.endm

.macro C_TSX
   and rsr, rsr, #0x7D		@ clear N,Z
   movs x, rsp, LSL #24		@ move SP to X
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_TXA
   and rsr, rsr, #0x7D		@ clear N,Z
   movs  acc, x			@ move X to Acc
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_TXS
   mov rsp, x, LSR #24		@ move X to SP (pre-shift to lower byte)
.endm

.macro C_TYA
   and rsr, rsr, #0x7D		@ clear N,Z
   movs  acc, y			@ move Y to Acc	
   orreq rsr, rsr, #0x02        @ set Z flag
   orrmi rsr, rsr, #0x80	@ set N flag
.endm

.macro C_WAI
   b done			@ endless loop until reset since no Interrupts yet
.endm

#
# end
#
